package com.factory.auth.domain.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.factory.auth.application.service.AuthService;
import com.factory.auth.domain.mapper.PermissionsMapper;
import com.factory.auth.domain.mapper.RolePermissionsMapper;
import com.factory.auth.domain.mapper.UserRolesMapper;
import com.factory.auth.domain.model.vo.token.TokenVal;
import com.factory.domain.model.aggregates.ResponseBean;
import com.factory.domain.model.aggregates.ResponseEnum;
import com.factory.infrastructure.constant.remote.RemoteCommonConstants;
import com.factory.infrastructure.constant.remote.RemoteRedisConstants;
import com.factory.infrastructure.po.remote.Permission;
import com.factory.infrastructure.po.remote.RolePermission;
import com.factory.infrastructure.po.remote.UserRole;
import com.factory.infrastructure.util.res.ResponseBeanBuilder;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.AntPathMatcher;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import java.util.List;
import java.util.stream.Collectors;

@Service
public class AuthServiceImpl implements AuthService {

    @Autowired
    private UserRolesMapper userRolesMapper;
    @Autowired
    private RolePermissionsMapper rolePermissionsMapper;
    @Autowired
    private PermissionsMapper permissionMapper;
    @Autowired
    private RedisTemplate<Object, Object> redisTemplate;

    private AntPathMatcher antMatcher = new AntPathMatcher();
    @Override
    public ResponseBean<Long> checkToken(String token) {
        if (StringUtils.isBlank(token)) {
            return ResponseBeanBuilder.fail(ResponseEnum.UNAUTHORIZED);
        }
        boolean res = redisTemplate.opsForHash().hasKey(RemoteRedisConstants.TOKEN_HASH_NAME, token);
        if (!res) {
            return ResponseBeanBuilder.fail(ResponseEnum.UNAUTHORIZED);
        }
        TokenVal tokenVal = (TokenVal) redisTemplate.opsForHash().get(RemoteRedisConstants.TOKEN_HASH_NAME, token);
        if (ObjectUtils.isEmpty(tokenVal)) {
            return ResponseBeanBuilder.fail(ResponseEnum.UNAUTHORIZED);
        }
        return ResponseBeanBuilder.ok(tokenVal.getUserId());
    }

    @Override
    public boolean checkPermission(String requestUrl, String client, String token, Long userId) {
        List<UserRole> userRoles = userRolesMapper.selectList(Wrappers.<UserRole>lambdaQuery().eq(UserRole::getUserId, userId));
        List<Long> roleIdList = userRoles.stream().map(UserRole::getRoleId).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(roleIdList)) {
            if (roleIdList.contains(RemoteCommonConstants.ROLE_ADMIN)) {
                return true;
            }
            List<RolePermission> rolePermissions = rolePermissionsMapper.selectList(Wrappers.<RolePermission>lambdaQuery().in(RolePermission::getRoleId, roleIdList));
            List<Long> permissionIds = rolePermissions.stream().map(RolePermission::getPermissionId).collect(Collectors.toList());
            if(CollUtil.isNotEmpty(permissionIds)){
                List<Permission> permissions = permissionMapper.selectBatchIds(permissionIds);
                boolean hasPermission = permissions.stream().map(Permission::getUrl).anyMatch(p->antMatcher.match(p, requestUrl));
                return hasPermission;
            }
        }
        return false;
    }

    @Override
    public ResponseBean<String> refreshToken(String refreshToken) {

        return null;
    }

}
