package com.factory.auth.domain.service;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.factory.auth.application.service.TokenService;
import com.factory.auth.application.service.UsersService;
import com.factory.auth.domain.mapper.UsersMapper;
import com.factory.auth.domain.model.vo.res.LoginVo;
import com.factory.auth.domain.model.vo.res.UserVo;
import com.factory.auth.domain.model.vo.token.TokenType;
import com.factory.auth.domain.model.vo.token.TokenVo;
import com.factory.auth.infrastructure.util.Base64;
import com.factory.domain.model.aggregates.ResponseBean;
import com.factory.domain.model.aggregates.ResponseEnum;
import com.factory.infrastructure.constant.remote.RemoteCommonConstants;
import com.factory.infrastructure.po.remote.Role;
import com.factory.infrastructure.po.remote.UserInfo;
import com.factory.infrastructure.util.res.ResponseBeanBuilder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
@Transactional
public class UsersServiceImpl extends ServiceImpl<UsersMapper, UserInfo> implements UsersService {

    @Autowired
    private TokenService tokenService;
	 

    @Override
    public ResponseBean<LoginVo> login(String client, String username, String password) {
        client = client.toUpperCase();
        String psd = Base64.encode(password.getBytes());
        UserVo userVo = this.baseMapper.getUserByUsername(username);
        if(BeanUtil.isEmpty(userVo)) {
           return ResponseBeanBuilder.fail(ResponseEnum.USERNAME_NOT_FOND.getMessage());
        }
        if (userVo.isLock()) {
           return ResponseBeanBuilder.fail(ResponseEnum.USER_STOP.getMessage());
        }
        if (!psd.equals(userVo.getPassword())) {
           return ResponseBeanBuilder.fail(ResponseEnum.PASSWORD_FILE.getMessage());
        }
        if (!RemoteCommonConstants.ADMIN.equals(userVo.getId())) {
            if (TokenType.SHORT_EXPIRED_TOKEN.toString().equals(client)) {
                int code = tokenService.checkPermission(userVo.getId(), "login", client);
                if (code != RemoteCommonConstants.NOT_DEL) {
                   return ResponseBeanBuilder.fail(ResponseEnum.USER_NOT_PERMISSION.getMessage());
                }
            }
        }
        TokenVo tokenVo = this.tokenService.getToken(client, userVo.getId());
        LoginVo loginVo = new LoginVo(userVo.getNo(), userVo.getUsername(), userVo.getDeptName(),
                tokenVo.getToken(), tokenVo.getRefreshToken(), userVo.getId(),"");
        
        List<Role> list = this.baseMapper.getUserRole(userVo.getId());
        String roleName = list.stream().map(r-> r.getName()).collect(Collectors.joining());
        loginVo.setRoleName(roleName);
        return ResponseBeanBuilder.ok(loginVo);
	}
	@Override
	public ResponseBean<LoginVo> login(String client, Long userId) {
		client = client.toUpperCase();
		UserVo userVo = this.baseMapper.getUserById(userId);

		if (BeanUtil.isEmpty(userVo)) {
			return ResponseBeanBuilder.fail(ResponseEnum.USERNAME_NOT_FOND.getMessage());
		}
		if (userVo.isLock()) {
			return ResponseBeanBuilder.fail(ResponseEnum.USER_STOP.getMessage());
		}
		if (!RemoteCommonConstants.ADMIN.equals(userVo.getId())) {
			if (TokenType.SHORT_EXPIRED_TOKEN.toString().equals(client)) {
				int code = tokenService.checkPermission(userVo.getId(), "login", client);
				if (code != RemoteCommonConstants.NOT_DEL) {
					return ResponseBeanBuilder.fail(ResponseEnum.USER_NOT_PERMISSION.getMessage());
				}
			}
		}
		TokenVo tokenVo = this.tokenService.getToken(client, userVo.getId());
		LoginVo loginVo = new LoginVo(userVo.getNo(), userVo.getUsername(), userVo.getDeptName(), tokenVo.getToken(),
				tokenVo.getRefreshToken(), userVo.getId(),"");
        
        List<Role> list = this.baseMapper.getUserRole(userVo.getId());
        String roleName = list.stream().map(r-> r.getName()).collect(Collectors.joining());
        loginVo.setRoleName(roleName);
		return ResponseBeanBuilder.ok(loginVo);
	}
}
